//+----------------------------------------------------------+
//|                     Ehlers fisher transform - double.mq4 |
//|                                                   mladen |
//+----------------------------------------------------------+
#property  copyright "mladen"
#property  link      "mladenfx@gmail.com"

#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  Gold
#property  indicator_color2  DeepSkyBlue
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_level1  0

//
//
//
//
//
 
extern int    period1          = 10;
extern int    period2          = 15;
extern int    price            = PRICE_MEDIAN;
extern bool   arrowsShow       = true;
extern string arrowsIdentifier = "fisherArrows";
extern color  arrowsUpColor    = DeepSkyBlue;
extern color  arrowsDnColor    = Red;

//
//
//
//
//

double buffer1[];
double buffer2[];
double Values1[];
double Values2[];
double Prices[];
double trend[];
   
//+----------------------------------------------------------+
//|                                                          |
//+----------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,buffer1);
      SetIndexBuffer(1,buffer2);
      SetIndexBuffer(2,Prices);
      SetIndexBuffer(3,Values1);
      SetIndexBuffer(4,Values2);
      SetIndexBuffer(5,trend);
   IndicatorShortName("Ehlers\' Fisher transform ("+period1+","+period2+")");
   return(0);
}
int deinit()
{
   if (arrowsShow) deleteArrows();
   return(0);
}


//+----------------------------------------------------------+
//|                                                          |
//+----------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit,counted_bars=IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
         
   for(i=limit; i>=0; i--)
   {  
      Prices[i] = iMA(NULL,0,1,0,MODE_SMA,price,i);
      
      //
      //
      //
      //
      //
                  
         double MaxH = Prices[ArrayMaximum(Prices,period1,i)];
         double MinL = Prices[ArrayMinimum(Prices,period1,i)];
         if (MaxH!=MinL)
               Values1[i] = 0.33*2*((Prices[i]-MinL)/(MaxH-MinL)-0.5)+0.67*Values1[i+1];
         else  Values1[i] = 0.00;
               Values1[i] = MathMin(MathMax(Values1[i],-0.999),0.999); 

         //
         //
         //
         //
         //
         
         MaxH = Prices[ArrayMaximum(Prices,period2,i)];
         MinL = Prices[ArrayMinimum(Prices,period2,i)];
         if (MaxH!=MinL)
               Values2[i] = 0.33*2*((Prices[i]-MinL)/(MaxH-MinL)-0.5)+0.67*Values2[i+1];
         else  Values2[i] = 0.00;
               Values2[i] = MathMin(MathMax(Values2[i],-0.999),0.999); 

      //
      //
      //
      //
      //

      buffer1[i] = 0.5*MathLog((1+Values1[i])/(1-Values1[i]))+0.5*buffer1[i+1];
      buffer2[i] = 0.5*MathLog((1+Values2[i])/(1-Values2[i]))+0.5*buffer2[i+1];
      
      if (arrowsShow)
      {
         deleteArrow(Time[i]);
         trend[i]   = trend[i+1];
         if (buffer1[i] < 0 && buffer2[i] < 0) trend[i] = -1;
         if (buffer1[i] > 0 && buffer2[i] > 0) trend[i] =  1;
         if (trend[i]!=trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
         }
      }      
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}
void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}